local L = LibStub( "AceLocale-3.0" ):GetLocale( "UnderHood" )
local UH = UnderHood
local AceEvent = LibStub( "AceEvent-3.0" )
local Threat = LibStub( "Threat-2.0" )
local _G = _G
local UnitExists = UnitExists
local UnitCanAttack = UnitCanAttack
local UnitGUID = UnitGUID

local Provider = {}
Provider.__index = Provider
Provider.name = "Threat"
Provider.title = L["Threat"]
Provider.supportedUnits = { "player" }
Provider.manageVisibility = true

function Provider:Create( bar, zone, config )
	local self = {}

	setmetatable( self, Provider )
	AceEvent:Embed( self )

	self.bar = bar
	self.zone = zone
	self.config = config

	self.playerGUID = UnitGUID( "player" )

	return self
end

function Provider:UnitSupported( unit )
	return unit == "player"
end

function Provider:IsBarVisible()
	return self.targetGUID and self.inCombat or false
end

function Provider:OnEnterCombat()
	self.inCombat = true
	self.bar:Show( self.zone )

	self:Update()
end

function Provider:OnLeaveCombat()
	self.inCombat = nil
	self.bar:Hide( self.zone )
end

function Provider:Connect()
	Threat.RegisterCallback( self, "ThreatUpdated", "ThreatUpdated" )
	self:RegisterEvent( "PLAYER_TARGET_CHANGED" )

	self:PLAYER_TARGET_CHANGED()
end

function Provider:Disconnect()
	self:UnregisterAllEvents()
	Threat.UnregisterCallback( self, "ThreatUpdated" );
end

function Provider:Update()
	local r,g,b = unpack( UH.db.profile.colors.unit.hostile )

	self.bar:SetColor( self.zone, r, g, b, 1 )
	self:SetValue()
end

function Provider:ThreatUpdated( event, srcGUID, dstGUID, threat )
	if self.targetGUID and dstGUID == self.targetGUID then
		if srcGUID == self.playerGUID or srcGUID == UnitGUID( "targettarget" ) then
			self:SetValue()
		end
	end
end

function Provider:PLAYER_TARGET_CHANGED()
	if UnitExists( "target" ) then
		self.targetGUID = UnitGUID( "target" )
	else
		self.targetGUID = nil
	end

	self:Update()
end

function Provider:SetValue()
	local threat = self.targetGUID and Threat:GetThreat( self.playerGUID, self.targetGUID ) or 0
	local maxThreat = self.targetGUID and Threat:GetMaxThreatOnTarget( self.targetGUID ) or 0

	if maxThreat == 0 then
		threat = 0
	else
		threat = threat/maxThreat
	end

	self.bar:SetValue( self.zone, threat )

--[[
	TODO: add tank detection and scaling/colorization
	local ps = self.config.providerSettings
	local max = ps and ps.maximum or 1.0
	local nThreat = threat/max
	
   	local r,g,b = unpack( UH.db.profile.colors.unit.hostile )
   	
	if ps and ps.colors and ps.colors ~= "static" then
		if ps.colors == "amount" then
		    if nThreat < 0.9 then
		    else if nThreat
		else
		end
	end
--]]
end

local maximums = {
	[100] = L["Normal (100%)"],
	[110] = L["Melee (110%)"],
	[130] = L["Ranged (130%)"],
	[150] = L["Tank (150%)"],
}

local colors = {
	static = L["Static threat color"],
	amount = L["Color by threat amount"],
	deficit = L["Color by threat deficit"],
}

--[[
function Provider:SetupOptions( options )
	options.maximum = {
		name = L["Maximum value"],
		type = "select",
		order = 100,
		values = maximums,
		get = function() return self.config.providerSettings and self.config.providerSettings.maximum or 100 end,
		set = function( info, value )
			if not self.config.providerSettings then
				self.config.providerSettings = {}
			end

			self.config.providerSettings.maximum = value
			self:Update()
		end,
	}

	options.color = {
		name = L["Color"],
		type = "select",
		order = 101,
		values = colors,
		get = function() return self.config.providerSettings and self.config.providerSettings.color or "static" end,
		set = function( info, value )
			if not self.config.providerSettings then
				self.config.providerSettings = {}
			end

			self.config.providerSettings.color = value
			self:Update()
		end,
	}
end
--]]

UnderHood_Bars:RegisterProvider( Provider )
